package java.lang;

import java.util.Formatter;
import java.util.Locale;
import java.util.Iterator;
import java.util.StringJoiner;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.Objects;
import java.nio.charset.Charset;
import java.io.UnsupportedEncodingException;
import java.util.Arrays;
import java.util.Comparator;
import java.io.ObjectStreamField;
import java.io.Serializable;

public final class String implements Serializable, Comparable<String>, CharSequence
{
    private final char[] value;
    private int hash;
    private static final long serialVersionUID = -6849794470754667710L;
    private static final ObjectStreamField[] serialPersistentFields;
    public static final Comparator<String> CASE_INSENSITIVE_ORDER;
    
    public String() {
        this.value = "".value;
    }
    
    public String(String s) {
        this.value = s.value;
        this.hash = s.hash;
    }
    
    public String(char[] original) {
        this.value = Arrays.copyOf(original, original.length);
    }
    
    public String(char[] original, int from, int n) {
        if (from < 0) {
            throw new StringIndexOutOfBoundsException(from);
        }
        if (n <= 0) {
            if (n < 0) {
                throw new StringIndexOutOfBoundsException(n);
            }
            if (from <= original.length) {
                this.value = "".value;
                return;
            }
        }
        if (from > original.length - n) {
            throw new StringIndexOutOfBoundsException(from + n);
        }
        this.value = Arrays.copyOfRange(original, from, from + n);
        return;
    }
    
    public String(int[] array, int i, int n) {
        if (i < 0) {
            throw new StringIndexOutOfBoundsException(i);
        }
        if (n <= 0) {
            if (n < 0) {
                throw new StringIndexOutOfBoundsException(n);
            }
            if (i <= array.length) {
                this.value = "".value;
                return;
            }
        }
        if (i > array.length - n) {
            throw new StringIndexOutOfBoundsException(i + n);
        }
        int n2 = i + n;
        int n3;
        for (; i < n2; ++i) {
            n3 = array[i];
            if (!Character.isBmpCodePoint(n3)) {
                if (Character.isValidCodePoint(n3)) {
                    ++n;
                }
                else {
                    throw new IllegalArgumentException(Integer.toString(n3));
                }
            }
        }
        char[] value = new char[n];
        for (int n4 = 0; i < n2; ++i, ++n4) {
            int n5 = array[i];
            if (Character.isBmpCodePoint(n5)) {
                value[n4] = (char)n5;
            }
            else {
                Character.toSurrogates(n5, value, n4++);
            }
        }
        this.value = value;
        return;
    }
    
    @Deprecated
    public String(byte[] array, int n, int n2, int n3) {
        checkBounds(array, n2, n3);
        char[] value = new char[n3];
        if (n == 0) {
            while (n3-- > 0) {
                value[n3] = (char)(array[n3 + n2] & 0xFF);
            }
        }
        else {
            n <<= 8;
            while (n3-- > 0) {
                value[n3] = (char)(n | (array[n3 + n2] & 0xFF));
            }
        }
        this.value = value;
    }
    
    @Deprecated
    public String(byte[] array, int n) {
        this(array, n, 0, array.length);
    }
    
    private static void checkBounds(byte[] array, int n, int n2) {
        if (n2 < 0) {
            throw new StringIndexOutOfBoundsException(n2);
        }
        if (n < 0) {
            throw new StringIndexOutOfBoundsException(n);
        }
        if (n > array.length - n2) {
            throw new StringIndexOutOfBoundsException(n + n2);
        }
        return;
    }
    
    public String(byte[] array, int n, int n2, String s) throws UnsupportedEncodingException {
        if (s == null) {
            throw new NullPointerException("charsetName");
        }
        checkBounds(array, n, n2);
        this.value = StringCoding.decode(s, array, n, n2);
        return;
    }
    
    public String(byte[] array, int n, int n2, Charset charset) {
        if (charset == null) {
            throw new NullPointerException("charset");
        }
        checkBounds(array, n, n2);
        this.value = StringCoding.decode(charset, array, n, n2);
        return;
    }
    
    public String(byte[] array, String s) throws UnsupportedEncodingException {
        this(array, 0, array.length, s);
    }
    
    public String(byte[] array, Charset charset) {
        this(array, 0, array.length, charset);
    }
    
    public String(byte[] array, int n, int n2) {
        checkBounds(array, n, n2);
        this.value = StringCoding.decode(array, n, n2);
    }
    
    public String(byte[] array) {
        this(array, 0, array.length);
    }
    
    public String(StringBuffer sb) {
        synchronized (sb) {
            this.value = Arrays.copyOf(sb.getValue(), sb.length());
        }
    }
    
    public String(StringBuilder sb) {
        this.value = Arrays.copyOf(sb.getValue(), sb.length());
    }
    
    String(char[] value, boolean b) {
        this.value = value;
    }
    
    @Override
    public int length() {
        return this.value.length;
    }
    
    public boolean isEmpty() {
        return this.value.length == 0;
    }
    
    @Override
    public char charAt(int n) {
        if (n < 0 || n >= this.value.length) {
            throw new StringIndexOutOfBoundsException(n);
        }
        return this.value[n];
    }
    
    public int codePointAt(int n) {
        if (n < 0 || n >= this.value.length) {
            throw new StringIndexOutOfBoundsException(n);
        }
        return Character.codePointAtImpl(this.value, n, this.value.length);
    }
    
    public int codePointBefore(int n) {
        int n2 = n - 1;
        if (n2 < 0 || n2 >= this.value.length) {
            throw new StringIndexOutOfBoundsException(n);
        }
        return Character.codePointBeforeImpl(this.value, n, 0);
    }
    
    public int codePointCount(int n, int n2) {
        if (n < 0 || n2 > this.value.length || n > n2) {
            throw new IndexOutOfBoundsException();
        }
        return Character.codePointCountImpl(this.value, n, n2 - n);
    }
    
    public int offsetByCodePoints(int n, int n2) {
        if (n < 0 || n > this.value.length) {
            throw new IndexOutOfBoundsException();
        }
        return Character.offsetByCodePointsImpl(this.value, 0, this.value.length, n, n2);
    }
    
    void getChars(char[] array, int n) {
        System.arraycopy(this.value, 0, array, n, this.value.length);
    }
    
    public void getChars(int n, int n2, char[] array, int n3) {
        if (n < 0) {
            throw new StringIndexOutOfBoundsException(n);
        }
        if (n2 > this.value.length) {
            throw new StringIndexOutOfBoundsException(n2);
        }
        if (n > n2) {
            throw new StringIndexOutOfBoundsException(n2 - n);
        }
        System.arraycopy(this.value, n, array, n3, n2 - n);
        return;
    }
    
    @Deprecated
    public void getBytes(int i, int n, byte[] obj, int n2) {
        if (i < 0) {
            throw new StringIndexOutOfBoundsException(i);
        }
        if (n > this.value.length) {
            throw new StringIndexOutOfBoundsException(n);
        }
        if (i > n) {
            throw new StringIndexOutOfBoundsException(n - i);
        }
        Objects.requireNonNull(obj);
        for (char[] value = this.value; i < n; obj[n2++] = (byte)value[i++]) {}
        return;
    }
    
    public byte[] getBytes(String s) throws UnsupportedEncodingException {
        if (s == null) {
            throw new NullPointerException();
        }
        return StringCoding.encode(s, this.value, 0, this.value.length);
    }
    
    public byte[] getBytes(Charset charset) {
        if (charset == null) {
            throw new NullPointerException();
        }
        return StringCoding.encode(charset, this.value, 0, this.value.length);
    }
    
    public byte[] getBytes() {
        return StringCoding.encode(this.value, 0, this.value.length);
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if ((o instanceof String)) {
            String s = (String)o;
            int length = this.value.length;
            if (length == s.value.length) {
                char[] value = this.value;
                char[] value2 = s.value;
                int n = 0;
                while (length-- != 0) {
                    if (value[n] != value2[n]) {
                        return false;
                    }
                    ++n;
                }
                return true;
            }
        }
        return false;
    }
    
    public boolean contentEquals(StringBuffer sb) {
        return this.contentEquals(sb);
    }
    
    private boolean nonSyncContentEquals(AbstractStringBuilder abstractStringBuilder) {
        char[] value = this.value;
        char[] value2 = abstractStringBuilder.getValue();
        int length = value.length;
        if (length != abstractStringBuilder.length()) {
            return false;
        }
        int i = 0;
        while (i < length) {
            if (value[i] != value2[i]) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    public boolean contentEquals(CharSequence charSequence) {
        if ((charSequence instanceof AbstractStringBuilder)) {
            if ((charSequence instanceof StringBuffer)) {
                synchronized (charSequence) {
                    return this.nonSyncContentEquals((AbstractStringBuilder)charSequence);
                }
            }
            return this.nonSyncContentEquals((AbstractStringBuilder)charSequence);
        }
        else {
            if ((charSequence instanceof String)) {
                return this.equals(charSequence);
            }
            char[] value = this.value;
            int length = value.length;
            if (length != charSequence.length()) {
                return false;
            }
            int i = 0;
            while (i < length) {
                if (value[i] != charSequence.charAt(i)) {
                    return false;
                }
                ++i;
            }
            return true;
        }
    }
    
    public boolean equalsIgnoreCase(String s) {
        return this == s || (s != null && s.value.length == this.value.length && this.regionMatches(true, 0, s, 0, this.value.length));
    }
    
    @Override
    public int compareTo(String s) {
        int length = this.value.length;
        int length2 = s.value.length;
        int min = Math.min(length, length2);
        char[] value = this.value;
        char[] value2 = s.value;
        int i = 0;
        while (i < min) {
            int n = value[i];
            int n2 = value2[i];
            if (n != n2) {
                return n - n2;
            }
            ++i;
        }
        return length - length2;
    }
    
    public int compareToIgnoreCase(String s) {
        return String.CASE_INSENSITIVE_ORDER.compare(this, s);
    }
    
    public boolean regionMatches(int n, String s, int n2, int n3) {
        char[] value = this.value;
        char[] value2 = s.value;
        if (n2 < 0 || n < 0 || (long)n > (long)this.value.length - (long)n3 || (long)n2 > (long)s.value.length - (long)n3) {
            return false;
        }
        while (n3-- > 0) {
            if (value[n++] == value2[n2++]) {
                continue;
            }
            return false;
        }
        return true;
    }
    
    public boolean regionMatches(boolean b, int n, String s, int n2, int n3) {
        char[] value = this.value;
        char[] value2 = s.value;
        if (n2 < 0 || n < 0 || (long)n > (long)this.value.length - (long)n3 || (long)n2 > (long)s.value.length - (long)n3) {
            return false;
        }
        while (n3-- > 0) {
            int n4 = value[n++];
            int n5 = value2[n2++];
            if (n4 == n5) {
                continue;
            }
            if (b) {
                int upperCase = Character.toUpperCase((char)n4);
                int upperCase2 = Character.toUpperCase((char)n5);
                if (upperCase == upperCase2) {
                    continue;
                }
                if (Character.toLowerCase((char)upperCase) == Character.toLowerCase((char)upperCase2)) {
                    continue;
                }
            }
            return false;
        }
        return true;
    }
    
    public boolean startsWith(String s, int n) {
        char[] value = this.value;
        char[] value2 = s.value;
        int n2 = 0;
        int length = s.value.length;
        if (n < 0 || n > this.value.length - length) {
            return false;
        }
        while (--length >= 0) {
            if (value[n++] == value2[n2++]) {
                continue;
            }
            return false;
        }
        return true;
    }
    
    public boolean startsWith(String s) {
        return this.startsWith(s, 0);
    }
    
    public boolean endsWith(String s) {
        return this.startsWith(s, this.value.length - s.value.length);
    }
    
    @Override
    public int hashCode() {
        int hash = this.hash;
        if (hash == 0 && this.value.length > 0) {
            char[] value = this.value;
            for (int i = 0; i < this.value.length; ++i) {
                hash = 31 * hash + value[i];
            }
            this.hash = hash;
        }
        return hash;
    }
    
    public int indexOf(int n) {
        return this.indexOf(n, 0);
    }
    
    public int indexOf(int n, int i) {
        int length = this.value.length;
        if (i < 0) {
            i = 0;
        }
        else if (i >= length) {
            return -1;
        }
        if (n < 65536) {
            char[] value = this.value;
            while (i < length) {
                if (value[i] == n) {
                    return i;
                }
                ++i;
            }
            return -1;
        }
        return this.indexOfSupplementary(n, i);
    }
    
    private int indexOfSupplementary(int n, int i) {
        if (Character.isValidCodePoint(n)) {
            char[] value = this.value;
            int highSurrogate = Character.highSurrogate(n);
            int lowSurrogate = Character.lowSurrogate(n);
            int n2 = value.length - 1;
            while (i < n2) {
                if (value[i] == highSurrogate && value[i + 1] == lowSurrogate) {
                    return i;
                }
                ++i;
            }
        }
        return -1;
    }
    
    public int lastIndexOf(int n) {
        return this.lastIndexOf(n, this.value.length - 1);
    }
    
    public int lastIndexOf(int n, int n2) {
        if (n < 65536) {
            char[] value = this.value;
            int i = Math.min(n2, value.length - 1);
            while (i >= 0) {
                if (value[i] == n) {
                    return i;
                }
                --i;
            }
            return -1;
        }
        return this.lastIndexOfSupplementary(n, n2);
    }
    
    private int lastIndexOfSupplementary(int n, int n2) {
        if (Character.isValidCodePoint(n)) {
            char[] value = this.value;
            int highSurrogate = Character.highSurrogate(n);
            int lowSurrogate = Character.lowSurrogate(n);
            int i = Math.min(n2, value.length - 2);
            while (i >= 0) {
                if (value[i] == highSurrogate && value[i + 1] == lowSurrogate) {
                    return i;
                }
                --i;
            }
        }
        return -1;
    }
    
    public int indexOf(String s) {
        return this.indexOf(s, 0);
    }
    
    public int indexOf(String s, int n) {
        return indexOf(this.value, 0, this.value.length, s.value, 0, s.value.length, n);
    }
    
    static int indexOf(char[] array, int n, int n2, String s, int n3) {
        return indexOf(array, n, n2, s.value, 0, s.value.length, n3);
    }
    
    static int indexOf(char[] array, int n, int n2, char[] array2, int n3, int n4, int n5) {
        if (n5 >= n2) {
            return (n4 == 0) ? n2 : -1;
        }
        if (n5 < 0) {
            n5 = 0;
        }
        if (n4 == 0) {
            return n5;
        }
        int n6 = array2[n3];
        for (int n7 = n + (n2 - n4), i = n + n5; i <= n7; ++i) {
            if (array[i] != n6) {
                while (++i <= n7 && array[i] != n6) {}
            }
            if (i <= n7) {
                int n8 = i + 1;
                int n9 = n8 + n4 - 1;
                for (int n10 = n3 + 1; n8 < n9 && array[n8] == array2[n10]; ++n8, ++n10) {}
                if (n8 == n9) {
                    return i - n;
                }
            }
        }
        return -1;
    }
    
    public int lastIndexOf(String s) {
        return this.lastIndexOf(s, this.value.length);
    }
    
    public int lastIndexOf(String s, int n) {
        return lastIndexOf(this.value, 0, this.value.length, s.value, 0, s.value.length, n);
    }
    
    static int lastIndexOf(char[] array, int n, int n2, String s, int n3) {
        return lastIndexOf(array, n, n2, s.value, 0, s.value.length, n3);
    }
    
    static int lastIndexOf(char[] array, int n, int n2, char[] array2, int n3, int n4, int n5) {
        int n6 = n2 - n4;
        if (n5 < 0) {
            return -1;
        }
        if (n5 > n6) {
            n5 = n6;
        }
        if (n4 == 0) {
            return n5;
        }
        int n7 = n3 + n4 - 1;
        int n8 = array2[n7];
        int n9 = n + n4 - 1;
        int n10 = n9 + n5;
    Label_0062:
        while (true) {
            if (n10 >= n9 && array[n10] != n8) {
                --n10;
            }
            else {
                if (n10 < n9) {
                    return -1;
                }
                int i = n10 - 1;
                int n11 = i - (n4 - 1);
                int n12 = n7 - 1;
                while (i > n11) {
                    if (array[i--] == array2[n12--]) {
                        continue;
                    }
                    --n10;
                    continue Label_0062;
                }
                return n11 - n + 1;
            }
        }
    }
    
    public String substring(int n) {
        if (n < 0) {
            throw new StringIndexOutOfBoundsException(n);
        }
        int n2 = this.value.length - n;
        if (n2 < 0) {
            throw new StringIndexOutOfBoundsException(n2);
        }
        return (n == 0) ? this : new String(this.value, n, n2);
    }
    
    public String substring(int n, int n2) {
        if (n < 0) {
            throw new StringIndexOutOfBoundsException(n);
        }
        if (n2 > this.value.length) {
            throw new StringIndexOutOfBoundsException(n2);
        }
        int n3 = n2 - n;
        if (n3 < 0) {
            throw new StringIndexOutOfBoundsException(n3);
        }
        return (n == 0 && n2 == this.value.length) ? this : new String(this.value, n, n3);
    }
    
    @Override
    public CharSequence subSequence(int n, int n2) {
        return this.substring(n, n2);
    }
    
    public String concat(String s) {
        int length = s.length();
        if (length == 0) {
            return this;
        }
        int length2 = this.value.length;
        char[] copy = Arrays.copyOf(this.value, length2 + length);
        s.getChars(copy, length2);
        return new String(copy, true);
    }
    
    public String replace(char c, char c2) {
        if (c != c2) {
            int length = this.value.length;
            int i = -1;
            char[] value = this.value;
            while (++i < length && value[i] != c) {}
            if (i < length) {
                char[] array = new char[length];
                for (int j = 0; j < i; ++j) {
                    array[j] = value[j];
                }
                while (i < length) {
                    int n = value[i];
                    array[i] = ((n == c) ? c2 : n);
                    ++i;
                }
                return new String(array, true);
            }
        }
        return this;
    }
    
    public boolean matches(String regex) {
        return Pattern.matches(regex, this);
    }
    
    public boolean contains(CharSequence charSequence) {
        return this.indexOf(charSequence.toString()) > -1;
    }
    
    public String replaceFirst(String regex, String replacement) {
        return Pattern.compile(regex).matcher(this).replaceFirst(replacement);
    }
    
    public String replaceAll(String regex, String replacement) {
        return Pattern.compile(regex).matcher(this).replaceAll(replacement);
    }
    
    public String replace(CharSequence charSequence, CharSequence charSequence2) {
        return Pattern.compile(charSequence.toString(), 16).matcher(this).replaceAll(Matcher.quoteReplacement(charSequence2.toString()));
    }
    
    public String[] split(String regex, int limit) {
        int n;
        if (((regex.value.length == 1 && ".$|()[{^?*+\\".indexOf(n = regex.charAt(0)) == -1) || (regex.length() == 2 && regex.charAt(0) == '\\' && ((n = regex.charAt(1)) - 48 | 57 - n) < 0 && (n - 97 | 122 - n) < 0 && (n - 65 | 90 - n) < 0)) && (n < '?' || n > '?')) {
            int length = 0;
            int n2 = limit > 0;
            ArrayList o = new ArrayList();
            int index;
            while ((index = this.indexOf(n, length)) != -1) {
                if (n2 == 0 || o.size() < limit - 1) {
                    o.add(this.substring(length, index));
                    length = index + 1;
                }
                else {
                    o.add(this.substring(length, this.value.length));
                    length = this.value.length;
                    break;
                }
            }
            if (length == 0) {
                return new String[] { this };
            }
            if (n2 == 0 || o.size() < limit) {
                o.add(this.substring(length, this.value.length));
            }
            int size = o.size();
            if (limit == 0) {
                while (size > 0 && ((String)o.get(size - 1)).length() == 0) {
                    --size;
                }
            }
            return (String[])o.subList(0, size).toArray(new String[size]);
        }
        else {
            return Pattern.compile(regex).split(this, limit);
        }
    }
    
    public String[] split(String s) {
        return this.split(s, 0);
    }
    
    public static String join(CharSequence charSequence, CharSequence... obj) {
        Objects.requireNonNull(charSequence);
        Objects.requireNonNull(obj);
        StringJoiner stringJoiner = new StringJoiner(charSequence);
        for (int length = obj.length, i = 0; i < length; ++i) {
            stringJoiner.add(obj[i]);
        }
        return stringJoiner.toString();
    }
    
    public static String join(CharSequence charSequence, Iterable<? extends CharSequence> obj) {
        Objects.requireNonNull(charSequence);
        Objects.requireNonNull(obj);
        StringJoiner stringJoiner = new StringJoiner(charSequence);
        Iterator<Object> iterator = obj.iterator();
        while (iterator.hasNext()) {
            stringJoiner.add((CharSequence)iterator.next());
        }
        return stringJoiner.toString();
    }
    
    public String toLowerCase(Locale locale) {
        if (locale == null) {
            throw new NullPointerException();
        }
        int length = this.value.length;
        int i = 0;
        while (i < length) {
            int n = this.value[i];
            Label_0099: {
                if (n >= '?' && n <= '?') {
                    int codePoint = this.codePointAt(i);
                    if (codePoint != Character.toLowerCase(codePoint)) {
                        break Label_0099;
                    }
                    i += Character.charCount(codePoint);
                }
                else {
                    if (n != Character.toLowerCase((char)n)) {
                        break Label_0099;
                    }
                    ++i;
                }
                continue;
            }
            char[] array = new char[length];
            int n2 = 0;
            System.arraycopy(this.value, 0, array, 0, i);
            String language = locale.getLanguage();
            int n3 = language == "tr" || language == "az" || language == "lt";
            int charCount;
            for (; i < length; i += charCount) {
                int codePoint2 = this.value[i];
                if ((char)codePoint2 >= '?' && (char)codePoint2 <= '?') {
                    codePoint2 = this.codePointAt(i);
                    charCount = Character.charCount(codePoint2);
                }
                else {
                    charCount = 1;
                }
                int n4;
                if (n3 || codePoint2 == 931 || codePoint2 == 304) {
                    n4 = ConditionalSpecialCasing.toLowerCaseEx(this, i, locale);
                }
                else {
                    n4 = Character.toLowerCase(codePoint2);
                }
                if (n4 == -1 || n4 >= 65536) {
                    char[] array2;
                    if (n4 == -1) {
                        array2 = ConditionalSpecialCasing.toLowerCaseCharArray(this, i, locale);
                    }
                    else {
                        if (charCount == 2) {
                            n2 += Character.toChars(n4, array, i + n2) - charCount;
                            continue;
                        }
                        array2 = Character.toChars(n4);
                    }
                    int length2 = array2.length;
                    if (length2 > charCount) {
                        char[] array3 = new char[array.length + length2 - charCount];
                        System.arraycopy(array, 0, array3, 0, i + n2);
                        array = array3;
                    }
                    for (int j = 0; j < length2; ++j) {
                        array[i + n2 + j] = array2[j];
                    }
                    n2 += length2 - charCount;
                }
                else {
                    array[i + n2] = (char)n4;
                }
            }
            return new String(array, 0, length + n2);
        }
        return this;
    }
    
    public String toLowerCase() {
        return this.toLowerCase(Locale.getDefault());
    }
    
    public String toUpperCase(Locale locale) {
        if (locale == null) {
            throw new NullPointerException();
        }
        int length = this.value.length;
        int i = 0;
        while (i < length) {
            int codePoint = this.value[i];
            int charCount;
            if (codePoint >= 55296 && codePoint <= 56319) {
                codePoint = this.codePointAt(i);
                charCount = Character.charCount(codePoint);
            }
            else {
                charCount = 1;
            }
            int upperCaseEx = Character.toUpperCaseEx(codePoint);
            if (upperCaseEx != -1 && codePoint == upperCaseEx) {
                i += charCount;
            }
            else {
                int n = 0;
                char[] array = new char[length];
                System.arraycopy(this.value, 0, array, 0, i);
                String language = locale.getLanguage();
                int n2 = language == "tr" || language == "az" || language == "lt";
                int charCount2;
                for (; i < length; i += charCount2) {
                    int codePoint2 = this.value[i];
                    if ((char)codePoint2 >= '?' && (char)codePoint2 <= '?') {
                        codePoint2 = this.codePointAt(i);
                        charCount2 = Character.charCount(codePoint2);
                    }
                    else {
                        charCount2 = 1;
                    }
                    int n3;
                    if (!(n2 == 0)) {
                        n3 = ConditionalSpecialCasing.toUpperCaseEx(this, i, locale);
                    }
                    else {
                        n3 = Character.toUpperCaseEx(codePoint2);
                    }
                    if (n3 == -1 || n3 >= 65536) {
                        char[] array2;
                        if (n3 == -1) {
                            if (!(n2 == 0)) {
                                array2 = ConditionalSpecialCasing.toUpperCaseCharArray(this, i, locale);
                            }
                            else {
                                array2 = Character.toUpperCaseCharArray(codePoint2);
                            }
                        }
                        else {
                            if (charCount2 == 2) {
                                n += Character.toChars(n3, array, i + n) - charCount2;
                                continue;
                            }
                            array2 = Character.toChars(n3);
                        }
                        int length2 = array2.length;
                        if (length2 > charCount2) {
                            char[] array3 = new char[array.length + length2 - charCount2];
                            System.arraycopy(array, 0, array3, 0, i + n);
                            array = array3;
                        }
                        for (int j = 0; j < length2; ++j) {
                            array[i + n + j] = array2[j];
                        }
                        n += length2 - charCount2;
                    }
                    else {
                        array[i + n] = (char)n3;
                    }
                }
                return new String(array, 0, length + n);
            }
        }
        return this;
    }
    
    public String toUpperCase() {
        return this.toUpperCase(Locale.getDefault());
    }
    
    public String trim() {
        int length;
        int n;
        char[] value;
        for (length = this.value.length, n = 0, value = this.value; n < length && value[n] <= ' '; ++n) {}
        while (n < length && value[length - 1] <= ' ') {
            --length;
        }
        return (n > 0 || length < this.value.length) ? this.substring(n, length) : this;
    }
    
    @Override
    public String toString() {
        return this;
    }
    
    public char[] toCharArray() {
        char[] array = new char[this.value.length];
        System.arraycopy(this.value, 0, array, 0, this.value.length);
        return array;
    }
    
    public static String format(String format, Object... args) {
        return new Formatter().format(format, args).toString();
    }
    
    public static String format(Locale l, String format, Object... args) {
        return new Formatter(l).format(format, args).toString();
    }
    
    public static String valueOf(Object o) {
        return (o == null) ? "null" : o.toString();
    }
    
    public static String valueOf(char[] array) {
        return new String(array);
    }
    
    public static String valueOf(char[] array, int n, int n2) {
        return new String(array, n, n2);
    }
    
    public static String copyValueOf(char[] array, int n, int n2) {
        return new String(array, n, n2);
    }
    
    public static String copyValueOf(char[] array) {
        return new String(array);
    }
    
    public static String valueOf(boolean b) {
        return b ? "true" : "false";
    }
    
    public static String valueOf(char c) {
        return new String(new char[] { c }, true);
    }
    
    public static String valueOf(int n) {
        return Integer.toString(n);
    }
    
    public static String valueOf(long n) {
        return Long.toString(n);
    }
    
    public static String valueOf(float n) {
        return Float.toString(n);
    }
    
    public static String valueOf(double n) {
        return Double.toString(n);
    }
    
    public native String intern();
    
    static {
        serialPersistentFields = new ObjectStreamField[0];
        CASE_INSENSITIVE_ORDER = new String.CaseInsensitiveComparator();
    }
    
    private static class CaseInsensitiveComparator implements Comparator<String>, Serializable
    {
        private static final long serialVersionUID = 8575799808933029326L;
        
        @Override
        public int compare(String s, String s2) {
            int length = s.length();
            int length2 = s2.length();
            for (int min = Math.min(length, length2), i = 0; i < min; ++i) {
                int char1 = s.charAt(i);
                int char2 = s2.charAt(i);
                if (char1 != char2) {
                    int upperCase = Character.toUpperCase((char)char1);
                    int upperCase2 = Character.toUpperCase((char)char2);
                    if (upperCase != upperCase2) {
                        int lowerCase = Character.toLowerCase((char)upperCase);
                        int lowerCase2 = Character.toLowerCase((char)upperCase2);
                        if (lowerCase != lowerCase2) {
                            return lowerCase - lowerCase2;
                        }
                    }
                }
            }
            return length - length2;
        }
        
        private Object readResolve() {
            return String.CASE_INSENSITIVE_ORDER;
        }
    }
}
